import numpy as np
from modshogun import RealFeatures
from modshogun import ConstMean
from modshogun import MulticlassLabels
from modshogun import CSVFile
from modshogun import GaussianProcessClassification
from modshogun import MulticlassAccuracy
from modshogun import SerializableAsciiFile
from modshogun import Math
from modshogun import GaussianKernel
from modshogun import WrappedObjectArray
from modshogun import MultiLaplaceInferenceMethod
from modshogun import SoftMaxLikelihood

f_feats_train = CSVFile("../../data/classifier_4class_2d_linear_features_train.dat")
f_feats_test = CSVFile("../../data/classifier_4class_2d_linear_features_test.dat")
f_labels_train = CSVFile("../../data/classifier_4class_2d_linear_labels_train.dat")
f_labels_test = CSVFile("../../data/classifier_4class_2d_linear_labels_test.dat")
Math.init_random(1)

#![create_features]
features_train = RealFeatures(f_feats_train)
features_test = RealFeatures(f_feats_test)
labels_train = MulticlassLabels(f_labels_train)
labels_test = MulticlassLabels(f_labels_test)
#![create_features]

#![create_appropriate_kernel_and_mean_function]
kernel = GaussianKernel(2.0)
mean_function = ConstMean()
#![create_appropriate_kernel_and_mean_function]

#![create_instance]
gauss_likelihood = SoftMaxLikelihood()
inference_method = MultiLaplaceInferenceMethod(kernel, features_train, mean_function, labels_train, gauss_likelihood)
gp_classifier = GaussianProcessClassification(inference_method)
#![create_instance]

#![train_and_apply]
gp_classifier.train()
labels_predict = gp_classifier.apply_multiclass(features_test)
#![train_and_apply]

#![extract_the_probabilities]
prob = gp_classifier.get_probabilities(features_train)
#![extract_the_probabilities]

#![evaluate_accuracy]
evals = MulticlassAccuracy()
accuracy = evals.evaluate(labels_predict, labels_test)
#![evaluate_accuracy]

# integration testing variables
output = labels_test.get_labels()

# Serialize output for integration testing (automatically generated)
__sg_storage = WrappedObjectArray()
__sg_storage_file = SerializableAsciiFile("gaussian_process_classifier.dat", 'w')
__sg_storage.append_wrapped_real_vector(prob, "prob")
__sg_storage.append_wrapped_real(accuracy, "accuracy")
__sg_storage.append_wrapped_real_vector(output, "output")
__sg_storage.save_serializable(__sg_storage_file)
